
#ifndef __ALGORITHMAN__LL1PARSERGENERATOR_TPP
#define __ALGORITHMAN__LL1PARSERGENERATOR_TPP

template<class token> ContextFreeGrammars::LL1ParserGenerator<token>::LL1ParserGenerator(const ContextFreeGrammar<token>& forcfg)
    : cfg(forcfg)
{
    for(std::vector<NonTerminal*>::iterator nt = cfg.nonterminals.begin(); nt != cfg.nonterminals.end(); nt++)
        CollectNonTerminalFirst1(*nt);

    for(std::vector<NonTerminal*>::iterator nt = cfg.nonterminals.begin(); nt != cfg.nonterminals.end(); nt++)
        CollectNonTerminalFollow1(*nt);
}

template<class token> ContextFreeGrammars::LL1ParserGenerator<token>::~LL1ParserGenerator()
{
}






template<class token> void ContextFreeGrammars::LL1ParserGenerator<token>::CollectNonTerminalFollow1(const NonTerminal *targetnt)
{
    std::set<const NonTerminal*> VisitedNonterminals;

    VisitedNonterminals.insert(targetnt);
    std::queue<const NonTerminal*> ntqueue;
    for(ntqueue.push(targetnt); !ntqueue.empty(); ntqueue.pop())
    {
        const NonTerminal* front = ntqueue.front();

        for(std::vector<NonTerminal*>::const_iterator n = cfg.nonterminals.begin(); n != cfg.nonterminals.end(); n++)
        {
            for(std::vector<Production*>::const_iterator p = (*n)->productions.begin(); p != (*n)->productions.end(); p++)
            {
                for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                {
                    if(*s != front)
                        continue;

                    bool EpsilonFollows = true;
                    std::vector<Symbol*>::const_iterator sfollow = s;
                    for(sfollow++ ; EpsilonFollows && sfollow != (*p)->produced.end(); sfollow++)
                    {
                        // insert the terminal
                        Terminal<token> *t = dynamic_cast<Terminal<token>*>(*sfollow);
                        if(t != NULL)
                        {
                            nonterminalfollow1[targetnt].insert(t);
                            EpsilonFollows = false;
                            break;
                        }

                        // insert all elements of the first1 set of the nonterminal
                        NonTerminal *nt = static_cast<NonTerminal*>(*sfollow);
                        typename std::map<const NonTerminal*, std::set<Terminal<token>*> >::const_iterator sourceset = nonterminalfirst1.find(nt);
                        bool ContainsEpsilon = false;
                        for(typename std::set<Terminal<token>*>::const_iterator f = sourceset->second.begin(); f != sourceset->second.end(); f++)
                            if(*f != NULL)
                                nonterminalfollow1[targetnt].insert(*f);
                            else
                                ContainsEpsilon = true;

                        EpsilonFollows = ContainsEpsilon;
                    }

                    if(EpsilonFollows && VisitedNonterminals.find(*n) == VisitedNonterminals.end())
                    {
                        ntqueue.push(*n);
                        VisitedNonterminals.insert(*n);

                        if(*n == cfg.startsymbol) // the symbol can be the last one
                            nonterminalfollow1[targetnt].insert(NULL);
                    }
                }
            }
        }

    }
}

template<class token> void ContextFreeGrammars::LL1ParserGenerator<token>::CollectNonTerminalFirst1(const NonTerminal *targetnt)
{
    std::set<const NonTerminal*> VisitedNonterminals;
    std::set<const NonTerminal*> epsProducers = cfg.FindEpsilonProducers();

    VisitedNonterminals.insert(targetnt);
    std::queue<const NonTerminal*> ntqueue;
    for(ntqueue.push(targetnt); !ntqueue.empty(); ntqueue.pop())
    {
        const NonTerminal* front = ntqueue.front();
        for(std::vector<Production*>::const_iterator p = front->productions.begin(); p != front->productions.end(); p++)
        {
            for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
            {
                Terminal<token> *t = dynamic_cast<Terminal<token>*>(*s);
                if(t != NULL)
                {
                    nonterminalfirst1[targetnt].insert(t);
                    break;
                }

                NonTerminal *nt = static_cast<NonTerminal*>(*s);
                if(VisitedNonterminals.find(nt) == VisitedNonterminals.end())
                {
                    VisitedNonterminals.insert(nt);
                    ntqueue.push(nt);
                }
                if(epsProducers.find(nt) == epsProducers.end())
                    break;
            }
        }
    }

    if(epsProducers.find(targetnt) != epsProducers.end())
        nonterminalfirst1[targetnt].insert(NULL);
}

template<class token> std::set<ContextFreeGrammars::Terminal<token>*> ContextFreeGrammars::LL1ParserGenerator<token>::first1(const std::vector<Symbol*>& w) const
{
    std::set<Terminal<token>*> result;

    bool EpsilonProducible = true;
    for(std::vector<Symbol*>::const_iterator i = w.begin(); EpsilonProducible && i != w.end(); i++)
    {
        Terminal<token> *t = dynamic_cast<Terminal<token>*>(*i);
        if(t != NULL)
        {
            result.insert(t);
            EpsilonProducible = false;
            break;
        }

        NonTerminal *nt = static_cast<NonTerminal*>(*i);
        bool ContainsEpsilon = false;

        typename std::map<const NonTerminal*, std::set<Terminal<token>*> >::const_iterator sourceset = nonterminalfirst1.find(nt);
        for(typename std::set<Terminal<token>*>::const_iterator f = sourceset->second.begin(); f != sourceset->second.end(); f++)
            if(*f != NULL)
                result.insert(*f);
            else
                ContainsEpsilon = true;

        if(!ContainsEpsilon)
            EpsilonProducible = false;
    }

    if(EpsilonProducible)
        result.insert(NULL);
    return result;
}

template<class token> std::set<ContextFreeGrammars::Terminal<token>*> ContextFreeGrammars::LL1ParserGenerator<token>::lookahead1(const ContextFreeGrammars::NonTerminal* A, const std::vector<ContextFreeGrammars::Symbol*>& beta) const
{
    std::set<Terminal<token>*> result = first1(beta);
    if(result.find(NULL) != result.end())
    {
        result.erase(NULL);
        typename std::map<const NonTerminal*, std::set<Terminal<token>*> >::const_iterator sourceset = nonterminalfollow1.find(A);
        for(typename std::set<Terminal<token>*>::iterator f = sourceset->second.begin(); f != sourceset->second.end(); f++)
            result.insert(*f);
    }
    return result;
}





template<class token> bool ContextFreeGrammars::LL1ParserGenerator<token>::IsLL1Grammar() const
{
    for(std::vector<NonTerminal*>::const_iterator nt = cfg.nonterminals.begin(); nt != cfg.nonterminals.end(); nt++)
    {
        for(std::vector<Production*>::const_iterator p1 = (*nt)->productions.begin(); p1 != (*nt)->productions.end(); p1++)
        {
            std::vector<Production*>::const_iterator p2 = p1;
            for(p2++; p2 != (*nt)->productions.end(); p2++)
            {
                std::set<Terminal<token>*> p1lookahead = lookahead1(*nt, (*p1)->produced);
                std::set<Terminal<token>*> p2lookahead = lookahead1(*nt, (*p2)->produced);

                for(typename std::set<Terminal<token>*>::iterator i = p1lookahead.begin(); i != p1lookahead.end(); i++)
                    if(p2lookahead.find(*i) != p2lookahead.end()) // *i is in the intersection of the lookahead-sets => not an LL(1) grammar
                        return false;
            }
        }
    }
    return true;
}




template<class token> ContextFreeGrammars::LL1Parser<token>* ContextFreeGrammars::LL1ParserGenerator<token>::Generate() const
{
    LL1Parser<token>* result = new LL1Parser<token>();

    for(std::vector<NonTerminal*>::const_iterator n = cfg.nonterminals.begin(); n != cfg.nonterminals.end(); n++)
    {
        for(std::vector<Production*>::const_iterator p = (*n)->productions.begin(); p != (*n)->productions.end(); p++)
        {
            std::set<Terminal<token>*> plookahead = lookahead1(*n, (*p)->produced);
            for(typename std::set<Terminal<token>*>::iterator la = plookahead.begin(); la != plookahead.end(); la++)
                result->lookaheadTable[*n][*la] = *p;
                
        }
    }

    result->startsymbol = cfg.startsymbol;
    return result;
}

#endif









