
#ifndef __ALGORITHMAN__LRPARSER_H
#define __ALGORITHMAN__LRPARSER_H

#include "ContextFreeGrammar.h"
#include <stack>

    namespace ContextFreeGrammars
    {

        template<class token> class LRParser;


        template<class token> class LRParserAction
        {
        public:
            LRParserAction();
            virtual ~LRParserAction();
            virtual void Execute(Tokenizer<token, 1>& tokenizer, LRParser<token>* context, std::ostream& messagestream);
        };



        template<class token> class LRParserActionShift : public LRParserAction<token>
        {
            int followingState;
        public:
            LRParserActionShift(int followingState);
            virtual ~LRParserActionShift();
            virtual void Execute(Tokenizer<token, 1>& tokenizer, LRParser<token>* context, std::ostream& messagestream);
        };



        template<class token> class LRParserActionReduce : public LRParserAction<token>
        {
            NonTerminal* nonterminal;
            Production* production;
        public:
            LRParserActionReduce(NonTerminal* nonterminal, Production* production);
            virtual ~LRParserActionReduce();
            virtual void Execute(Tokenizer<token, 1>& tokenizer, LRParser<token>* context, std::ostream& messagestream);
        };



        template<class token> class LRParser
        {
            friend class LRParserAction<token>;
            friend class LRParserActionShift<token>;
            friend class LRParserActionReduce<token>;
        protected:
            std::stack<int> stateStack;
            //std::stack<Symbol*> parseStack;
            std::stack<void*> parseStack;

        public:
            int state;
            std::vector< std::map<token, LRParserAction<token>*> > actionTable;
            std::vector< LRParserAction<token>* > finalActionTable;
            std::vector< std::map<NonTerminal*, int> > jumpTable;

            LRParser();
            virtual ~LRParser();
            //void GenerateCPlusPlusCode(std::ostream& os);
            void* Parse(Tokenizer<token, 1>& tokenizer, std::ostream& messagestream);
        };

    }

#include "LRParser.tpp"

#endif
