
#ifndef __ALGORITHMAN__LRPARSER_TPP
#define __ALGORITHMAN__LRPARSER_TPP

template<class Token> ContextFreeGrammars::LRParser<Token>::LRParser()
{
}

template<class Token> ContextFreeGrammars::LRParser<Token>::~LRParser()
{
}

template<class Token> void* ContextFreeGrammars::LRParser<Token>::Parse(ContextFreeGrammars::Tokenizer<Token, 1>& tokenizer, std::ostream& messagestream)
{
    state = 0; // start state

    while(state >= 0 && state < actionTable.size())
    {
        LRParserAction<Token>* action;
        if(tokenizer.HasToken(0))
        {
            Token lookahead = tokenizer.PeekToken(0);
            typename std::map<Token,LRParserAction<Token>*>::iterator actionit = actionTable[state].find(lookahead);
            if(actionit == actionTable[state].end())
                action = NULL;
            else
                action = actionit->second;
        }
        else
        {
            action = finalActionTable[state];
        }

        if(action != NULL)
            action->Execute(tokenizer, this, messagestream);
        else
            break;
    }

    return NULL;
}




template<class Token> ContextFreeGrammars::LRParserAction<Token>::LRParserAction()
{
}

template<class Token> ContextFreeGrammars::LRParserAction<Token>::~LRParserAction()
{
}

template<class Token> void ContextFreeGrammars::LRParserAction<Token>::Execute(ContextFreeGrammars::Tokenizer<Token, 1>& tokenizer, ContextFreeGrammars::LRParser<Token>* context, std::ostream& messagestream)
{
    messagestream << "Terminate" << std::endl;
    messagestream.flush();
    context->state = -1;
}




template<class Token> ContextFreeGrammars::LRParserActionShift<Token>::LRParserActionShift(int followingState)
    : LRParserAction<Token>()
{
    this->followingState = followingState;
}

template<class Token> ContextFreeGrammars::LRParserActionShift<Token>::~LRParserActionShift()
{
}

template<class Token> void ContextFreeGrammars::LRParserActionShift<Token>::Execute(ContextFreeGrammars::Tokenizer<Token, 1>& tokenizer, ContextFreeGrammars::LRParser<Token>* context, std::ostream& messagestream)
{
    messagestream << "Shift " << followingState << " \"";
    Token tok = tokenizer.ConsumeToken();
    messagestream << tok << "\"" << std::endl;
    messagestream.flush();

    /*
    Terminal<Token>* t = new Terminal<Token>(tok);
    context->parseStack.push(t);
    */
    context->parseStack.push(NULL);

    context->stateStack.push(context->state);
    context->state = followingState;
}




template<class Token> ContextFreeGrammars::LRParserActionReduce<Token>::LRParserActionReduce(ContextFreeGrammars::NonTerminal* nonterminal, ContextFreeGrammars::Production* production)
    : LRParserAction<Token>()
{
    this->nonterminal = nonterminal;
    this->production = production;
}

template<class Token> ContextFreeGrammars::LRParserActionReduce<Token>::~LRParserActionReduce()
{
}

template<class Token> void ContextFreeGrammars::LRParserActionReduce<Token>::Execute(ContextFreeGrammars::Tokenizer<Token, 1>& tokenizer, ContextFreeGrammars::LRParser<Token>* context, std::ostream& messagestream)
{
    messagestream << "Reduce " << nonterminal->Name << " ->";
    messagestream.flush();
    for(int i = 0; i < production->produced.size(); i++)
        messagestream << " " << production->produced[i]->Name;
    messagestream << std::endl;
    messagestream.flush();

    for(int i = 0; i < production->produced.size(); i++)
    {
        void* s = context->parseStack.top();
        if(s != NULL)
            delete s;
        context->parseStack.pop();
    }
    context->parseStack.push(NULL);

    for(int i = 0; i < production->produced.size(); i++)
    {
        context->state = context->stateStack.top();
        context->stateStack.pop();
    }
    context->stateStack.push(context->state);
    context->state = context->jumpTable[context->state][this->nonterminal];
}

#endif







